@@ -1,4 +1,5 @@
-use crate::{Result, config::Config};
+use crate::Result;
+use crate::hook_options::HookOptions;
 
 mod commit_msg;
 mod pre_commit;
@@ -6,16 +7,18 @@ mod pre_push;
 mod prepare_commit_msg;
 
 /// Run a hook
-#[derive(Debug, clap::Args)]
+#[derive(clap::Args)]
 #[clap(visible_alias = "r", verbatim_doc_comment)]
 pub struct Run {
     #[clap(subcommand)]
     command: Option<Commands>,
     #[clap(hide = true)]
     other: Option<String>,
+    #[clap(flatten)]
+    hook: HookOptions,
 }
 
-#[derive(Debug, clap::Subcommand)]
+#[derive(clap::Subcommand)]
 enum Commands {
     CommitMsg(commit_msg::CommitMsg),
     PreCommit(pre_commit::PreCommit),
@@ -26,7 +29,7 @@ enum Commands {
 impl Run {
     pub async fn run(self) -> Result<()> {
         if let Some(hook) = &self.other {
-            return self.other(hook).await;
+            return self.hook.run(hook).await;
         }
         match self.command.unwrap() {
             Commands::CommitMsg(cmd) => cmd.run().await,
@@ -35,11 +38,4 @@ impl Run {
             Commands::PrepareCommitMsg(cmd) => cmd.run().await,
         }
     }
-
-    async fn other(&self, hook: &str) -> Result<()> {
-        let config = Config::get()?;
-        config
-            .run_hook(true, hook, &[], Default::default(), None, None)
-            .await
-    }
 }
